Package: csaw
Version: 1.38.0
Date: 2023-11-29
Title: ChIP-Seq Analysis with Windows
Authors@R: 
    c(person("Aaron", "Lun", role=c("aut", "cre"), email = "infinite.monkeys.with.keyboards@gmail.com"),
    person("Gordon", "Smyth", role="aut"))
Depends: R (>= 3.5.0), GenomicRanges, SummarizedExperiment
Imports: Rcpp, Matrix, BiocGenerics, Rsamtools, edgeR, limma, methods,
        S4Vectors, IRanges, GenomeInfoDb, stats, BiocParallel, metapod,
        utils
Suggests: AnnotationDbi, org.Mm.eg.db,
        TxDb.Mmusculus.UCSC.mm10.knownGene, testthat, GenomicFeatures,
        GenomicAlignments, knitr, BiocStyle, rmarkdown, BiocManager
LinkingTo: Rhtslib, zlibbioc, Rcpp
biocViews: MultipleComparison, ChIPSeq, Normalization, Sequencing,
        Coverage, Genetics, Annotation, DifferentialPeakCalling
Description: 
    Detection of differentially bound regions in ChIP-seq data with sliding windows,
    with methods for normalization and proper FDR control.
License: GPL-3
NeedsCompilation: yes
SystemRequirements: C++11, GNU make
VignetteBuilder: knitr
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/csaw
git_branch: RELEASE_3_19
git_last_commit: 8000717
git_last_commit_date: 2024-04-30
Repository: Bioconductor 3.19
Date/Publication: 2024-04-30
Packaged: 2024-05-01 03:19:20 UTC; biocbuild
Author: Aaron Lun [aut, cre],
  Gordon Smyth [aut]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
Built: R 4.4.1; x86_64-apple-darwin20.6.0; 2024-07-12 09:12:34 UTC; unix
