# Generated by roxygen2: do not edit by hand

export(asDGEList)
export(calculateCPM)
export(checkBimodality)
export(clusterFDR)
export(clusterWindows)
export(clusterWindowsList)
export(combineOverlaps)
export(combineTests)
export(consolidateOverlaps)
export(consolidateTests)
export(consolidateWindows)
export(controlClusterFDR)
export(correlateReads)
export(csawUsersGuide)
export(detailRanges)
export(empiricalFDR)
export(empiricalOverlaps)
export(extractReads)
export(filterWindows)
export(filterWindowsControl)
export(filterWindowsGlobal)
export(filterWindowsLocal)
export(filterWindowsProportion)
export(findMaxima)
export(findOverlapsList)
export(getBestOverlaps)
export(getBestTest)
export(getPESizes)
export(getWidths)
export(maximizeCcf)
export(mergeResults)
export(mergeResultsList)
export(mergeWindows)
export(mergeWindowsList)
export(minimalTests)
export(mixedClusters)
export(mixedOverlaps)
export(mixedTests)
export(normFactors)
export(normOffsets)
export(overlapResults)
export(overlapResultsList)
export(profileSites)
export(readParam)
export(reform)
export(regionCounts)
export(scaleControlFilter)
export(scaledAverage)
export(strandedCounts)
export(summitOverlaps)
export(upweightSummit)
export(windowCounts)
export(wwhm)
exportClasses(readParam)
exportMethods("$")
exportMethods(asDGEList)
exportMethods(reform)
exportMethods(show)
import(GenomicRanges)
import(SummarizedExperiment)
import(methods)
importClassesFrom(BiocParallel,BiocParallelParam)
importClassesFrom(GenomicRanges,GRanges)
importClassesFrom(GenomicRanges,GenomicRanges)
importClassesFrom(Rsamtools,BamFile)
importClassesFrom(SummarizedExperiment,RangedSummarizedExperiment)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(BiocGenerics,"start<-")
importFrom(BiocGenerics,"strand<-")
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,order)
importFrom(BiocGenerics,path)
importFrom(BiocGenerics,start)
importFrom(BiocGenerics,strand)
importFrom(BiocGenerics,width)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpisup)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,bpnworkers)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(GenomeInfoDb,"seqlengths<-")
importFrom(GenomeInfoDb,"seqlevels<-")
importFrom(GenomeInfoDb,Seqinfo)
importFrom(GenomeInfoDb,seqinfo)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqlevels)
importFrom(GenomeInfoDb,seqnames)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(IRanges,IRanges)
importFrom(IRanges,countOverlaps)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,flank)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,pintersect)
importFrom(IRanges,promoters)
importFrom(IRanges,ranges)
importFrom(IRanges,trim)
importFrom(Matrix,colSums)
importFrom(Rcpp,sourceCpp)
importFrom(Rsamtools,BamFile)
importFrom(Rsamtools,index)
importFrom(Rsamtools,scanBamHeader)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,I)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,queryLength)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,split)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(edgeR,DGEList)
importFrom(edgeR,addPriorCount)
importFrom(edgeR,aveLogCPM)
importFrom(edgeR,aveLogCPM.DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(edgeR,cpm)
importFrom(edgeR,mglmOneGroup)
importFrom(edgeR,scaleOffset)
importFrom(edgeR,scaleOffset.default)
importFrom(limma,loessFit)
importFrom(metapod,countGroupedDirection)
importFrom(metapod,groupedHolmMin)
importFrom(metapod,groupedSimes)
importFrom(metapod,summarizeGroupedDirection)
importFrom(methods,is)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,spline)
importFrom(stats,weighted.mean)
importFrom(utils,browseURL)
useDynLib(csaw, .registration=TRUE, .fixes="cxx_")
