---
title: Introducing the csaw package
author: 
- name: Aaron Lun
  affiliation: Walter and Eliza Hall Institute for Medical Research, Melbourne, Australia
date: "Revised: 17 February 2019"
output:
  BiocStyle::html_document
package: csaw 
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}    
---

```{r, echo=FALSE, results="hide"}
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Introduction

The `r Biocpkg("csaw")` package is designed for the _de novo_ detection of differentially bound regions from ChIP-seq data. 
It uses a sliding window approach to count reads across the genome from sorted and indexed BAM files. 
Each window is then tested for significant differences between libraries, using the methods in the `r Biocpkg("edgeR")` package. 
It implements statistical methods for:

- normalization of window counts between libraries
- independent filtering of uninteresting windows
- controlling the false discovery rate across aggregated windows 

`r Biocpkg("csaw")` can be applied to any data set containing multiple conditions with biological replication.
While intended for ChIP-seq data, the methods in this package can also be applied to any type of sequencing data where changes in genomic coverage are of interest.

# Documentation

The full user's guide is available as part of the online documentation in the `r Biocbook("csawBook")` package.
It can be obtained by typing:

```{r}
library(csaw)
if (interactive()) csawUsersGuide()
```

Documentation for speicific functions is available through the usual R help system, e.g., `?windowCounts`.
Further questions about the package should be directed to the [Bioconductor support site](https://support.bioconductor.org).

# Session information

```{r}
sessionInfo()
```
